from typing import Dict, Any, Optional

from ..Attributes import Email, Password, TemporaryToken

from ..Basic import BasicModel

from . import Responses


class Base(BasicModel):
    _prefix = "/v1/admin"
    _include_in_schema = False
    _response_module = Responses


class CreateUser(Base, Email, Password):
    admin: bool = False
    rate_limit: int = 30
    monthly_limit: int = 300
    company: str = None
    name: str = None


class DisableUser(Base, Email):
    pass


class GetTemporaryToken(Base, Email):
    pass


class ConfirmTemporaryToken(Base, Email, TemporaryToken):
    pass


class ActivateUser(Base, Email):
    pass


class ChangePasswordUser(Base, Email, Password):
    pass


class CreateStripeCustomer(Base, Email):
    pass


class ModifyUser(Base, Email):
    params: Dict[str, Any]


class AddUserStripePayment(Base, Email):
    pass


class GetUsersRequests(Base):
    filters: Dict[str, Any] = {}
    requests_fields: Dict[str, Any] = {}
    limit: int = 20
    skip: int = 0


class GetCustomerPaymentMethods(Base, Email):
    pass


class DetachCustomerPaymentMethod(Base):
    payment_method_id: str


class SetCustomerDefaultPaymentMethod(Base, Email):
    payment_method_id: str


class GetStripeCustomerInfo(Base, Email):
    pass


class GetCustomerInvoices(Base, Email):
    limit: int = 10
    started_after: Optional[str] = None
    ending_before: Optional[str] = None
    status: Optional[str] = None
