import re

from pydantic import BaseModel


class BasicModel(BaseModel):
    _response_module = None
    _response_error = None
    _include_in_schema = True
    _prefix = ""
    _method = "POST"

    @classmethod
    def endpoint(cls):
        name = cls.__name__
        s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
        return cls._prefix + "/" + re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()

    @classmethod
    def get_response_model(cls):
        return getattr(cls._response_module, cls.__name__)

    @classmethod
    def route(cls):
        return {
            "path": cls.endpoint(),
            "response_model": cls.get_response_model(),
            "response_model_skip_defaults": True,
            "include_in_schema": cls._include_in_schema
        }

    def call(self, client):
        body = {k: v for k, v in self.dict().items() if v is not None}
        return self.get_response_model()(**client.call(endpoint=self.endpoint(), body=body, method=self._method))

    @classmethod
    def fill(cls, **params):
        return cls(**{k: v for k, v in params.items() if v is not None and k != "self"})
