from typing import Optional

from ..Attributes import Properties, Precision, \
    Normalizations, ConceptsProperties, Lang, Split
from ..Basic import BasicModel
from pydantic import AnyUrl
from . import Responses


class Base(BasicModel):
    _prefix = "/v1/nlp"
    _response_module = Responses


class SemanticAnalysis(Base, Lang, Properties, ConceptsProperties, Precision, Normalizations, Split):
    text: str


class UrlAnalysis(Base, Properties, ConceptsProperties, Precision, Normalizations, Split):
    url: AnyUrl


#
# class DomainAnalysis(Base, Domain, Properties, Cursor, Precision):
#     _response_model = Response.DomainAnalysis

# class Tokenize(Base, Text, Lang, Normalize):
#     _response_model = Response.Tokenize


class DetectLang(Base):
    text: str


class Summarize(Base, Normalizations, Precision, Split):
    text: str


class TextsSimilarity(Base, Normalizations, Precision, Split):
    text1: str
    lang1: Optional[str]
    text2: str
    lang2: Optional[str]
