from datetime import datetime
import requests
from .ApiModels.User import Login
from lazy import lazy
from requests import HTTPError

from .Nlp import Nlp
from .User import User


class Client:

    def __init__(self, username=None, password=None, token=None, url="https://neutralnews.fr/api"):
        self._url = url
        self._username = username
        self._password = password
        self._token = token
        self._token_exp = None
        self._decode_token()
        self._get_token()
        self._decode_token()

    def _decode_token(self):
        import jwt
        if self._token:
            payload = jwt.decode(self._token, verify=False)
            self._username = payload.pop('sub')
            self._token_exp = datetime.fromtimestamp(payload.pop('exp'))
            self.__dict__.update({f"token_{k}": v for k, v in payload.items()})

    def _get_token(self):
        if self._token is None or self._token_exp is None or self._token_exp < datetime.utcnow():
            if self._password is None:
                raise HTTPError(details="Token expired", status_code=401)
            res = self._call_endpoint(url=f"{self._url}{Login.endpoint()}",
                                      method="GET", auth=(self._username, self._password))
            self._token = res['access_token']
        return self._token

    @staticmethod
    def _call_endpoint(url, method, auth=None, headers=None, json=None):
        res = requests.request(method=method, url=url, auth=auth,
                               headers=headers, json=json)
        try:
            res.raise_for_status()
        except Exception as e:
            detail = res.json().get("detail")
            if detail:
                raise type(e)(str(e) + ": " + detail)
            raise e
        return res.json()

    def call(self, endpoint, body, method):
        return self._call_endpoint(url=f'{self._url}{endpoint}', method=method,
                                   headers={'Authorization': 'Bearer ' + self._get_token()}, json=body)

    @lazy
    def nlp(self):
        return Nlp(self)

    @lazy
    def user(self):
        return User(self)
